@@ -26,7 +26,7 @@
 #ifdef _WIN32
 #include <malloc.h>
 #define alloca _alloca
-#elif !defined(__FreeBSD__)
+#elif !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__DragonFly__)
 #include <alloca.h>
 #endif
 #include <algorithm>
