@@ -662,8 +662,17 @@ Events::drop_schema_events(THD *thd, const char *db)
   */
   if (event_queue)
     event_queue->drop_schema_events(thd, &db_lex);
-  db_repository->drop_schema_events(thd, &db_lex);
-
+  if (db_repository)
+    db_repository->drop_schema_events(thd, &db_lex);
+  else
+  {
+    if ((db_repository= new Event_db_repository))
+    {
+      db_repository->drop_schema_events(thd, &db_lex);
+      delete db_repository;
+      db_repository= 0;
+    }
+  }
   DBUG_VOID_RETURN;
 }
 
@@ -711,15 +720,16 @@ send_show_create_event(THD *thd, Event_timed *et, Protocol *protocol)
 
   field_list.push_back(new (mem_root)
                        Item_empty_string(thd, "character_set_client",
-                                         MY_CS_NAME_SIZE), mem_root);
+                                         MY_CS_CHARACTER_SET_NAME_SIZE),
+                                         mem_root);
 
   field_list.push_back(new (mem_root)
                        Item_empty_string(thd, "collation_connection",
-                                         MY_CS_NAME_SIZE), mem_root);
+                                         MY_CS_COLLATION_NAME_SIZE), mem_root);
 
   field_list.push_back(new (mem_root)
                        Item_empty_string(thd, "Database Collation",
-                                         MY_CS_NAME_SIZE), mem_root);
+                                         MY_CS_COLLATION_NAME_SIZE), mem_root);
 
   if (protocol->send_result_set_metadata(&field_list,
                             Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
@@ -732,14 +742,11 @@ send_show_create_event(THD *thd, Event_timed *et, Protocol *protocol)
   protocol->store(tz_name->ptr(), tz_name->length(), system_charset_info);
   protocol->store(show_str.ptr(), show_str.length(),
                   et->creation_ctx->get_client_cs());
-  protocol->store(et->creation_ctx->get_client_cs()->csname,
-                  strlen(et->creation_ctx->get_client_cs()->csname),
+  protocol->store(&et->creation_ctx->get_client_cs()->cs_name,
                   system_charset_info);
-  protocol->store(et->creation_ctx->get_connection_cl()->name,
-                  strlen(et->creation_ctx->get_connection_cl()->name),
+  protocol->store(&et->creation_ctx->get_connection_cl()->coll_name,
                   system_charset_info);
-  protocol->store(et->creation_ctx->get_db_cl()->name,
-                  strlen(et->creation_ctx->get_db_cl()->name),
+  protocol->store(&et->creation_ctx->get_db_cl()->coll_name,
                   system_charset_info);
 
   if (protocol->write())
