@@ -209,7 +209,7 @@ void log_t::attach(log_file_t file, os_offset_t size)
 #ifdef HAVE_PMEM
   ut_ad(!buf);
   ut_ad(!flush_buf);
-  if (size && !(size_t(size) & 4095))
+  if (size && !(size_t(size) & 4095) && srv_operation != SRV_OPERATION_BACKUP)
   {
     void *ptr= log_mmap(log.m_file, size);
     if (ptr != MAP_FAILED)
@@ -220,6 +220,8 @@ void log_t::attach(log_file_t file, os_offset_t size)
 #if defined __linux__ || defined _WIN32
       set_block_size(CPU_LEVEL1_DCACHE_LINESIZE);
 #endif
+      log_maybe_unbuffered= true;
+      log_buffered= false;
       return;
     }
   }
@@ -231,18 +233,11 @@ void log_t::attach(log_file_t file, os_offset_t size)
 #endif
 
 #if defined __linux__ || defined _WIN32
-  if (!block_size)
-    set_block_size(512);
-# ifdef __linux__
-  else if (srv_file_flush_method != SRV_O_DSYNC &&
-           srv_file_flush_method != SRV_O_DIRECT &&
-           srv_file_flush_method != SRV_O_DIRECT_NO_FSYNC)
-    sql_print_information("InnoDB: Buffered log writes (block size=%u bytes)",
-                          block_size);
-#endif
-  else
-    sql_print_information("InnoDB: File system buffers for log"
-                          " disabled (block size=%u bytes)", block_size);
+  sql_print_information("InnoDB: %s (block size=%u bytes)",
+                        log_buffered
+                        ? "Buffered log writes"
+                        : "File system buffers for log disabled",
+                        block_size);
 #endif
 
 #ifdef HAVE_PMEM
@@ -260,16 +255,18 @@ void log_t::header_write(byte *buf, lsn_t lsn, bool encrypted)
   mach_write_to_4(my_assume_aligned<4>(buf) + LOG_HEADER_FORMAT,
                   log_sys.FORMAT_10_8);
   mach_write_to_8(my_assume_aligned<8>(buf + LOG_HEADER_START_LSN), lsn);
-  static constexpr const char LOG_HEADER_CREATOR_CURRENT[]=
-    "MariaDB "
-    IB_TO_STR(MYSQL_VERSION_MAJOR) "."
-    IB_TO_STR(MYSQL_VERSION_MINOR) "."
-    IB_TO_STR(MYSQL_VERSION_PATCH);
 
-  strcpy(reinterpret_cast<char*>(buf) + LOG_HEADER_CREATOR,
-         LOG_HEADER_CREATOR_CURRENT);
-  static_assert(LOG_HEADER_CREATOR_END - LOG_HEADER_CREATOR >=
-                sizeof LOG_HEADER_CREATOR_CURRENT, "compatibility");
+#if defined __GNUC__ && __GNUC__ > 7
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
+  strncpy(reinterpret_cast<char*>(buf) + LOG_HEADER_CREATOR,
+          "MariaDB " PACKAGE_VERSION,
+          LOG_HEADER_CREATOR_END - LOG_HEADER_CREATOR);
+#if defined __GNUC__ && __GNUC__ > 7
+# pragma GCC diagnostic pop
+#endif
+
   if (encrypted)
     log_crypt_write_header(buf + LOG_HEADER_CREATOR_END);
   mach_write_to_4(my_assume_aligned<4>(508 + buf), my_crc32c(0, buf, 508));
@@ -347,7 +344,7 @@ void log_t::close_file()
       ib::fatal() << "closing ib_logfile0 failed: " << err;
 }
 
-/** Acquire the latches that protect log resizing. */
+/** Acquire all latches that protect the log. */
 static void log_resize_acquire()
 {
   if (!log_sys.is_pmem())
@@ -361,7 +358,7 @@ static void log_resize_acquire()
   log_sys.latch.wr_lock(SRW_LOCK_CALL);
 }
 
-/** Release the latches that protect log resizing. */
+/** Release the latches that protect the log. */
 void log_resize_release()
 {
   log_sys.latch.wr_unlock();
@@ -375,6 +372,34 @@ void log_resize_release()
   }
 }
 
+#if defined __linux__ || defined _WIN32
+/** Try to enable or disable file system caching (update log_buffered) */
+void log_t::set_buffered(bool buffered)
+{
+  if (!log_maybe_unbuffered || is_pmem() || high_level_read_only)
+    return;
+  log_resize_acquire();
+  if (!resize_in_progress() && is_opened() && bool(log_buffered) != buffered)
+  {
+    os_file_close_func(log.m_file);
+    log.m_file= OS_FILE_CLOSED;
+    std::string path{get_log_file_path()};
+    log_buffered= buffered;
+    bool success;
+    log.m_file= os_file_create_func(path.c_str(),
+                                    OS_FILE_OPEN, OS_FILE_NORMAL, OS_LOG_FILE,
+                                    false, &success);
+    ut_a(log.m_file != OS_FILE_CLOSED);
+    sql_print_information("InnoDB: %s (block size=%u bytes)",
+                          log_buffered
+                          ? "Buffered log writes"
+                          : "File system buffers for log disabled",
+                          block_size);
+  }
+  log_resize_release();
+}
+#endif
+
 /** Start resizing the log and release the exclusive latch.
 @param size  requested new file_size
 @return whether the resizing was started successfully */
